// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "slistman.hpp"
#include <qimage.h>

SListMan::SListMan() {}

QString SListMan::next() {
  refresh();
  if (list.count()) {	//Still readable files there?
      QString lastFile , newFile = file;	//Effects that there aren't 2 loops necessary to detect that last listfile is'nt readable
      uint newPos = pos;
      do {
        newPos = QMIN(list.count() - 1 , newPos + 1);
        lastFile = newFile;
        newFile = list[newPos];
      } while (!QImageIO::imageFormat(newFile) and lastFile != newFile);	//Test for really readable files
      if (lastFile != newFile) {	//Found new file
        pos = newPos;	//Set name and pos of found readable file
        file = newFile;
      }
      return file;
    } else
      return (QString) 0;
}

QString SListMan::prev() {
  refresh();
  if (list.count()) {	//Still readable files there?
      QString lastFile , newFile = file;	//Effects that there aren't 2 loops necessary to detect that last listfile is'nt readable
      uint newPos = pos;
      do {
        newPos = QMAX(0 , (int) newPos - 1);
        lastFile = newFile;
        newFile = list[newPos];
      } while (!QImageIO::imageFormat(newFile) and lastFile != newFile);	//Test for really readable files
      if (lastFile != newFile) {
        pos = newPos;	//Set name and pos of found readable file
        file = newFile;
      }
      return file;
    } else
      return (QString) 0;
}

QString SListMan::last() {
  refresh();
  if (list.count()) {
      uint newPos = list.count();
      QString lastFile , newFile = dir.entryList().last();
      while (!QImageIO::imageFormat(newFile) and lastFile != newFile) {
	newPos = QMAX(0 , (int) newPos - 1);
	lastFile = newFile;
	newFile = list[newPos];
      }
      if (lastFile != newFile) {	//Found readable file?
        pos = newPos;	//Set name and pos of found readable file
        file = newFile;
      }
      return file;
    } else
      return (QString) 0;
}

QString SListMan::first() {
  refresh();
  if (list.count()) {
      uint newPos = 0;
      QString lastFile , newFile = list.first();
      while (!QImageIO::imageFormat(newFile) and lastFile != newFile) {
	newPos = QMIN(list.count() - 1 , newPos + 1);
	lastFile = newFile;
	newFile = list[newPos];
      }
      if (lastFile != newFile) {	//Found readable file?
        pos = newPos;	//Set name and pos of found readable file
        file = newFile;
      }
      return file;
    } else
      return (QString) 0;
}

void SListMan::refresh() {
  if (mode == directory) {
    dir.refresh();
    list.clear();
    const QString pathCache = dir.absPath() + "/";
    for (uint i = 0 ; i < dir.entryList().count() ; i++)
      list.append(pathCache + dir[i]);
  }
}

void SListMan::setOneMode() {
  dir = (QDir) 0;
  file = (QString) 0;
  list.clear();
  mode = one;
}

void SListMan::setDirectoryMode(QString dirFile) {
  file = dirFile;
  dir = QDir(file.left(file.findRev("/" , -2)) , "*.[bB][mM][pP] *.[gG][iI][fF] *.[mM][nN][gG] *.[jJ][pP][gG] *.[jJ][pP][eE][gG] *.[pP][bB][mM] *.[pP][gG][mM] *.[pP][nN][gG] *.[pP][pP][mM] *.[xX][bB][mM] *.[xX][pP][mM]");	//aktuelles Verzeichnis ermitteln
  dir.setSorting(QDir::Name);
  mode = directory;
  refresh();
  pos = list.findIndex(file);
}

void SListMan::setOwnMode(QStringList ownList) {
  list = ownList;
  mode = own;
  file = first();	//Call SListMan::first() to find first readable file
}

QString SListMan::browseInfo() {
  if (mode == one)
      return (QString) 0;
    else {
      const char *MODECHARS[] = {"1" , "O" , "D"};
      return QString("%1: %2/%3").arg(MODECHARS[mode]).arg(pos + 1).arg(list.count());
  }
}

bool SListMan::isBrowseable() {
  refresh();
  return !list.isEmpty();
}

QString SListMan::atActPos() {
  refresh();
  pos = QMAX(0 , QMIN((int) list.count() - 1 , pos));
  file = list[pos];
  if (list.count()) {	//Still readable files there?
      QString lastFile , newFile = file;	//Effects that there aren't 2 loops necessary to detect that last listfile is'nt readable
      uint newPos = pos;
      while (!QImageIO::imageFormat(newFile) and lastFile != newFile) {	//Test for really readable files
        newPos = QMIN(list.count() - 1 , newPos + 1);
        lastFile = newFile;
        newFile = list[newPos];
      }
      if (lastFile != newFile) {
        pos = newPos;	//Set name and pos of found readable file
        file = newFile;
      }
      return file;
    } else
      return (QString) 0;
}

void SListMan::convertToOwn() {
  if (mode == directory) {
    refresh();
    mode = own;
  }
}

QStringList SListMan::filelist() {
  if (mode != one) {
      refresh();
      return list;
    } else
      return QStringList();	//to avoid warning message of compiler
}

QString SListMan::goTo(const int gotoPos) {
  if (gotoPos == pos)
    return atActPos();
  QString newFile;
  if (pos > gotoPos)    //Go to a file BEFORE the current
      do
        newFile = prev();
      while (pos > gotoPos);
    else        //Go to a file AFTER the current
      do
        newFile = next();
      while (pos < gotoPos);
  return newFile;
}
  
