// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "spic.hpp"
#include <qpainter.h>
#include <qcursor.h>
#include "comps/scimgtools.hpp"

SPic::SPic(QWidget* parent) : QWidget(parent , "inhalt") {
  setMouseTracking(true);	//mouseMoveEvents auch, wenn kein Button gedr�ckt
  setCursor(Qt::CrossCursor);
  zoom = 1.0;
  mark = new SMark(this);
  connect(mark , SIGNAL(clicked(SEditReg)) , this , SLOT(registerClick(SEditReg)));
  show();
}

void SPic::paintEvent(QPaintEvent*) {
  bildrect.setRect(qRound((width() - drawdaten.width()) / 2.0) , qRound((height() - drawdaten.height()) / 2.0) , drawdaten.width() , drawdaten.height());
  QPainter(this).drawTiledPixmap(bildrect , drawdaten);
  mark->setGeometry(qRound(markX * zoom) + bildrect.x() , qRound(markY * zoom) + bildrect.y() , qRound(markB * zoom) , qRound(markH * zoom));
}

void SPic::mousePressEvent(QMouseEvent *maus) {
  if (!drawdaten.isNull())	//Anzeige vorhanden?
    switch (mark->mode) {
      case none :
	fixX = mapX(maus->x());
	fixY = mapY(maus->y());
	mark->setPaletteBackgroundColor(oppAvColor(&bilddaten , bilddaten.width() * bilddaten.height()));
	mark->setGeometry(maus->x() , maus->y() , 0 , 0);
	mark->mode = editE;
	mark->show();
	break;
      case editE :
      case editV :
      case editH :
      case editA :
	if (markB and markH) {	//Markierte Fl�che != 0?
	    mark->mode = fixed;
	    mark->update();
	  } else
	    killMark();
	break;
      case fixed :
	if (mark->geometry().contains(maus->pos()))
	  if (maus->button() == Qt::LeftButton or maus->button() == Qt::MidButton) {
	      QPoint punkt = mark->mapFromParent(maus->pos());
	      divX = qRound(punkt.x() / zoom);
	      divY = qRound(punkt.y() / zoom);
	      mark->mode = editA;
	      mark->update();
	    } else
	      killMark();
    }
}

void SPic::mouseMoveEvent(QMouseEvent *maus) {
  if (!drawdaten.isNull()) {	//Anzeige vorhanden?
    int x = mapX(maus->x()) , y = mapY(maus->y());
    switch (mark->mode) {
      case editE :	//Beliebige Ecke wird ge�ndert?
	if (x > fixX) {
	    markX = fixX;
	    markB = x - fixX;
	  } else {
	    markX = x;
	    markB = fixX - x;
	}
	if (y > fixY) {
	    markY = fixY;
	    markH = y - fixY;
	  } else {
	    markY = y;
	    markH = fixY - y;
	}
	break;
      case editV: 	//Beliebige vertikale Kante wird ge�ndert?
	if (x > fixX)
	    markB = x - fixX;
	  else {
	    markB = fixX - x;
	    markX = x;
	}
	break;
      case editH :	//Beliebige horizontale Kante wird ge�ndert?
	if (y > fixY)
	    markH = y - fixY;
	  else {
	    markY = y;
	    markH = fixY - y;
	}
	break;
      case editA :
	markX = QMAX(QMIN(x - divX , bilddaten.width() - markB) , 0);
	markY = QMAX(QMIN(y - divY , bilddaten.height() - markH) , 0);
      default :
	;
    }
    if (mark->mode != fixed and mark->mode != none) {
      mark->setGeometry(qRound(markX * zoom) + bildrect.x() , qRound(markY * zoom) + bildrect.y() , qRound(markB * zoom) , qRound(markH * zoom));
      emit markChanged4P(QString::number(markB) + char(215) + QString::number(markH) + "+" + QString::number(markX) + "+" + QString::number(markY) + " (" + QString::number(double(markH) / double(markB)) + "), " + QString::number(markH * markB));
    }
    x = mapX(maus->x() , false);
    y = mapY(maus->y() , false);
    if (bilddaten.valid(x , y)) {	//Um unzul�ssigen Pixeln entgegenzuwirken
	QRgb farbe = bilddaten.pixel(x , y);
	QString text(QString::number(x) + char(215) + QString::number(y) + ", " + QString::number(qRed(farbe)) + "-" + QString::number(qGreen(farbe)) + "-" + QString::number(qBlue(farbe)));
	if (bilddaten.depth() == 32)
	  text.append("-" + QString::number(qAlpha(farbe)));
	emit MPIChanged4P(text);
      } else
	emit MPIChanged4P(0);
  }
}

void SPic::leaveEvent(QEvent*) {
  emit MPIChanged4P(0);
}

void SPic::registerClick(SEditReg editReg) {
  switch (editReg) {
    case ML :	//mitte-links
      fixX = markB + markX;
      mark->mode = editV;
      break;
    case UM :	//unten-mitte
      fixY = markY;
      mark->mode = editH;
      break;
    case MR :	//mitte-rechts
      fixX = markX;
      mark->mode = editV;
      break;
    case OM :	//oben-mitte
      fixY = markH + markY;
      mark->mode = editH;
      break;
    case UR :	//unten-rechts
      fixX = markX;
      fixY = markY;
      mark->mode = editE;
      break;
    case OL :	//oben-links
      fixX = markB + markX;
      fixY = markH + markY;
      mark->mode = editE;
      break;
    case UL :	//unten-links
      fixX = markB + markX;
      fixY = markY;
      mark->mode = editE;
      break;
    case OR :	//oben-rechts
      fixX = markX;
      fixY = markH + markY;
      mark->mode = editE;
  }
}

int SPic::mapY(const int y , const bool grenzen) const {
  if (grenzen)
      return QMAX(QMIN(int((y - bildrect.y()) / zoom) , bilddaten.height()) , 0);
    else {
      double temp = (y - bildrect.y()) / zoom;
      if (temp >= 0.0)
	  return (int) temp;
	else
	  return -1;
  }
}

int SPic::mapX(const int x , const bool grenzen) const {
  if (grenzen)
      return QMAX(QMIN(int((x - bildrect.x()) / zoom) , bilddaten.width()) , 0);
    else {
      double temp = (x - bildrect.x()) / zoom;
      if (temp >= 0.0)
	  return (int) temp;
	else
	  return -1;
  }
}
