// Copyright (C) 2005-2007 Stefan Bauer <yoltia-maintain@freenet.de>

// This file is part of Yoltia.
//
// Yoltia is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License Version 2
// as published by the Free Software Foundation.
//    
// Yoltia is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Yoltia; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

#include "sslider.hpp"
#include <qapplication.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qimage.h>
#include <qcursor.h>
#include <qsettings.h>
#include "comps/scimgtools.hpp"

#include <cstdlib>

SSlider::SSlider(QStringList list) : QWidget(0 , "slider" , WDestructiveClose | WStyle_Customize | WStyle_NoBorder) {
  setCursor(Qt::BlankCursor);
  setPaletteBackgroundColor(QColor(0 , 0 , 0));
  filelist = list;
  desktopSize = QApplication::desktop()->screenGeometry(this).size();
  resize(desktopSize);
  show();
  currPos = UINT_MAX;
  nextSlide();
  QSettings set;
  if (set.readBoolEntry("/Yoltia/ss/timer" , true)) {
    connect(&timer , SIGNAL(timeout()) , this , SLOT(nextSlide()));
    timer.start(set.readNumEntry("/Yoltia/ss/time" , 10000));
  }
}

void SSlider::nextSlide() {
  QSettings set;
  if (set.readBoolEntry("/Yoltia/ss/random" , false)) {
      srand(time(0));
      slide(rand() % filelist.count());
    } else
      if (currPos + 1 < filelist.count())
          slide(currPos + 1);
        else
          if (set.readBoolEntry("/Yoltia/ss/repeat" , false))
	    slide(0);

}

void SSlider::slide(const uint pos) {
  QImage pic(filelist[pos]);
  pic = pic.smoothScale(desktopSize , QImage::ScaleMin);
  pic.setAlphaBuffer(false);
  int left = qRound((desktopSize.width() - pic.width()) / 2.0) , top = qRound((desktopSize.height() - pic.height()) / 2.0);
  QPainter painter(this);
  QPixmap pix(pic);	//Avoid little hang between drawing the rects and the image in cause of QImage - QPixmap conversion
  if (top == 0 and left > 0) {
    painter.fillRect(0 , 0 , left , pic.height() , QColor(0 , 0 , 0));
    painter.fillRect(pic.width() + left , 0 , left , pic.height() , QColor(0 , 0 , 0));
  }
  if (left == 0 and top > 0) {
    painter.fillRect(0 , 0 , pic.width() , top , QColor(0 , 0 , 0));
    painter.fillRect(0 , pic.height() + top , pic.width() , top , QColor(0 , 0 , 0));
  }
  painter.drawTiledPixmap(left , top , pic.width() , pic.height() , pix);
  QSettings set;
  painter.setPen(oppAvColor(&pic , desktopSize.width() * desktopSize.height()));
  if (set.readBoolEntry("/Yoltia/ss/displayFilename"))
    painter.drawText(0 , QApplication::font().pointSize() , filelist[pos] , -1);
  if (set.readBoolEntry("/Yoltia/ss/displayListpos"))
    painter.drawText(0 , QApplication::font().pointSize() * 2 + 1, QString("%1/%2").arg(pos + 1).arg(filelist.count()) , -1);
  currPos = pos;
}

void SSlider::mousePressEvent(QMouseEvent *) {
  timer.stop();
  nextSlide();
  QSettings set;
  if (set.readBoolEntry("/Yoltia/ss/timer" , true))
    timer.start(set.readNumEntry("/Yoltia/ss/time" , 10000));
}
